/****************************************************
Image ActiveX
Soubor: Objekt.h
(C) 2005 Radek Chalupa - www.radekchalupa.cz
*****************************************************/

#pragma once

#include "resource.h"
#include <atlctl.h>
#include "ImageAx.h"

// ImageAx
class ATL_NO_VTABLE ImageAx : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CStockPropImpl<ImageAx, IImageAx>,
	public IPersistStreamInitImpl<ImageAx>,
	public IOleControlImpl<ImageAx>,
	public IOleObjectImpl<ImageAx>,
	public IOleInPlaceActiveObjectImpl<ImageAx>,
	public IViewObjectExImpl<ImageAx>,
	public IOleInPlaceObjectWindowlessImpl<ImageAx>,
	public ISupportErrorInfo,
	public IPersistStorageImpl<ImageAx>,
	public ISpecifyPropertyPagesImpl<ImageAx>,
	public IQuickActivateImpl<ImageAx>,
	public IDataObjectImpl<ImageAx>,
	public IProvideClassInfo2Impl<&CLSID_ImageAx,
	NULL, &LIBID_ImageAxLib>,
	public CComCoClass<ImageAx, &CLSID_ImageAx>,
	public CComControl<ImageAx>
{
private:
	Bitmap* bitmap;
	CComBSTR pripona;
	bool pruhlednePozadi;
	CComBSTR soubor;

public:
	ImageAx()
	{
		bitmap = NULL;
		pruhlednePozadi = false;
		m_clrBackColor = GetSysColor(COLOR_3DFACE);
	}

DECLARE_OLEMISC_STATUS(OLEMISC_RECOMPOSEONRESIZE | 
	OLEMISC_CANTLINKINSIDE | 
	OLEMISC_INSIDEOUT | 
	OLEMISC_ACTIVATEWHENVISIBLE | 
	OLEMISC_SETCLIENTSITEFIRST
)

DECLARE_REGISTRY_RESOURCEID(IDR_IMAGEAX1)

BEGIN_COM_MAP(ImageAx)
	COM_INTERFACE_ENTRY(IImageAx)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(ImageAx)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	PROP_ENTRY("BackColor",
		DISPID_BACKCOLOR, CLSID_StockColorPage)
END_PROP_MAP()


BEGIN_MSG_MAP(ImageAx)
	CHAIN_MSG_MAP(CComControl<ImageAx>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IImageAx,
		};

		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IImageAx
public:
HRESULT OnDraw(ATL_DRAWINFO& di)
{
	RECT& rc = *(RECT*)di.prcBounds;
	HRGN hRgnOld = NULL;
	if (GetClipRgn(di.hdcDraw, hRgnOld) != 1)
		hRgnOld = NULL;
	bool bSelectOldRgn = false;
	HRGN hRgnNew = CreateRectRgn(rc.left,
		rc.top, rc.right, rc.bottom);
	if (hRgnNew != NULL)
		bSelectOldRgn = (SelectClipRgn(
			di.hdcDraw, hRgnNew) != ERROR);
	if (bitmap)
		KreslitBitmapu(di.hdcDraw, rc);
	else
		KreslitObjekt(di.hdcDraw, rc);

	if (bSelectOldRgn)
		SelectClipRgn(di.hdcDraw, hRgnOld);
	if (hRgnNew)
		if (!DeleteObject(hRgnNew))
			return E_FAIL;
	return S_OK;
}

OLE_COLOR m_clrBackColor;
void OnBackColorChanged()
{
	ATLTRACE(_T("OnBackColorChanged\n"));
}

DECLARE_PROTECT_FINAL_CONSTRUCT()

HRESULT FinalConstruct()
{
	return S_OK;
}

void FinalRelease() 
{
	Zavrit();
}

bool __forceinline KreslitObjekt(HDC hdc, RECT& rect)
{
	HBRUSH hbPozadi = NULL;
	if (!pruhlednePozadi)
	{
		COLORREF crPozadi;
		OleTranslateColor(m_clrBackColor, NULL, &crPozadi);
		hbPozadi = CreateSolidBrush(crPozadi);
		FillRect(hdc, &rect, hbPozadi);
	}
	SetBkMode(hdc, TRANSPARENT);
	DrawText(hdc, L"Image ActiveX", -1,
		&rect, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
	if (hbPozadi)
		if (!DeleteObject(hbPozadi))
			AtlThrowLastWin32();
	return true;
}

bool __forceinline KreslitBitmapu(HDC hdc, RECT& rect)
{
	if (!bitmap)
		return false;
	if (bitmap->GetHeight() == 0)
		return false;
	if (bitmap->GetWidth() == 0)
		return false;
		
	int sirkaRect = rect.right - rect.left;
	int vyskaRect = rect.bottom - rect.top;
	int xPoz = rect.left;
	int yPoz = rect.top;
	int sirkaObrCil = bitmap->GetWidth();
	int vyskaObrCil = bitmap->GetHeight();

	HBRUSH hbPozadi = NULL;
	RECT rcPozadi;
	if (!pruhlednePozadi)
	{
		COLORREF crPozadi;
		OleTranslateColor(m_clrBackColor, NULL, &crPozadi);
		hbPozadi = CreateSolidBrush(crPozadi);
	}

	Graphics graphics(hdc);
	// otestovat zda je men v obou rozmrech
	if (sirkaObrCil < sirkaRect && vyskaObrCil < vyskaRect)
	{
		xPoz = rect.left + ((sirkaRect - sirkaObrCil) / 2);
		yPoz = rect.top + ((vyskaRect - vyskaObrCil) / 2);
		if (hbPozadi)
		{
			// horn pruh
			rcPozadi.left = rect.left;
			rcPozadi.top = rect.top;
			rcPozadi.right = rect.right;
			rcPozadi.bottom = yPoz;
			FillRect(hdc, &rcPozadi, hbPozadi);
			// doln pruh
			rcPozadi.top = yPoz + vyskaObrCil;
			rcPozadi.bottom = rect.bottom;
			FillRect(hdc, &rcPozadi, hbPozadi);
			// lev pruh
			rcPozadi.left = rect.left;
			rcPozadi.top = yPoz;
			rcPozadi.right = xPoz;
			rcPozadi.bottom = yPoz + vyskaObrCil;
			FillRect(hdc, &rcPozadi, hbPozadi);
			// prav
			rcPozadi.left = xPoz + sirkaObrCil;
			rcPozadi.top = yPoz;
			rcPozadi.right = rect.right;
			rcPozadi.bottom = yPoz + vyskaObrCil;
			FillRect(hdc, &rcPozadi, hbPozadi);
		}
	}
	else
	{
		// nen li obrzek pesn do velikosti prvku
		if (sirkaRect < sirkaObrCil || vyskaRect < vyskaObrCil)
		{
			if ((float)sirkaRect/(float)sirkaObrCil >
				(float)vyskaRect/(float)vyskaObrCil)
			{ // obrzek "na vku", pruhy pozad vlevo a vpravo
				sirkaObrCil = (bitmap->GetWidth() *
					vyskaRect) / bitmap->GetHeight();
				vyskaObrCil = vyskaRect;
				xPoz = rect.left + ((sirkaRect - sirkaObrCil) / 2);
				if (hbPozadi)
				{
					rcPozadi.left = rect.left;
					rcPozadi.top = rect.top;
					rcPozadi.right = xPoz;
					rcPozadi.bottom = rect.bottom;
					FillRect(hdc, &rcPozadi, hbPozadi);
					// prav pruh pozad
					rcPozadi.right = rect.right;
					rcPozadi.left = xPoz + sirkaObrCil;
					FillRect(hdc, &rcPozadi, hbPozadi);
				}
			}
			else
			{ // obrzek na ku, pruhy pozad nahoe a dole
				vyskaObrCil = (bitmap->GetHeight() * sirkaRect) / bitmap->GetWidth();
				sirkaObrCil = sirkaRect;
				yPoz = rect.top + ((vyskaRect - vyskaObrCil) / 2);
				if (hbPozadi)
				{
					rcPozadi.left = rect.left;
					rcPozadi.right = rect.right;
					rcPozadi.top = rect.top;
					rcPozadi.bottom = yPoz;
					FillRect(hdc, &rcPozadi, hbPozadi);
					rcPozadi.top = yPoz + vyskaObrCil;
					rcPozadi.bottom = rect.bottom;
					FillRect(hdc, &rcPozadi, hbPozadi);
				}
			}
		}
		else
		{
			sirkaObrCil = sirkaRect;
			vyskaObrCil = vyskaRect;
		}
	}
	Rect rectCilObr(xPoz, yPoz,	sirkaObrCil, vyskaObrCil);
	if (hbPozadi)
		if (!DeleteObject(hbPozadi))
			throw 0;
	return graphics.DrawImage(bitmap,
		rectCilObr,
		0,
		0,
		bitmap->GetWidth(),
		bitmap->GetHeight(),
		UnitPixel) == Ok;
}

bool KreslitBitmap(HDC hDC, int xPoz, int yPoz,
	int sirka, int vyska, bool bZachovatPomer = true)
{
	if (!bitmap )
		return false;
	if (bitmap->GetHeight() == 0)
		return false;
	if (bitmap->GetWidth() == 0)
		return false;
	if (bZachovatPomer)
	{
		if ((UINT)sirka < bitmap->GetWidth() ||
			(UINT)vyska < bitmap->GetHeight())
		{
			if ( (float)sirka/(float)bitmap->GetWidth() >
				(float)vyska/(float)bitmap->GetHeight() )
				sirka = (bitmap->GetWidth()*vyska ) /
				bitmap->GetHeight();
			else
				vyska = (bitmap->GetHeight()*sirka ) /
				bitmap->GetWidth();
		}
		else
		{
			sirka = this->bitmap->GetWidth();
			vyska = this->bitmap->GetHeight();
		}
	}
	Graphics graphics(hDC);
	Rect rectCil(xPoz, yPoz, sirka, vyska);
	return graphics.DrawImage(bitmap,
		rectCil,
		0, 0,
		bitmap->GetWidth(),
		bitmap->GetHeight(),
		UnitPixel) == Ok;
}

STDMETHOD(Nacist)(BSTR soubor)
{
	this->soubor = CComBSTR(soubor);
	wchar_t* pPripona = PathFindExtensionW(soubor);
	if (pPripona)
		this->pripona = CComBSTR(pPripona+1);
	else
		this->pripona = CComBSTR(L"");
	if (this->bitmap)
	{
		delete this->bitmap;
		this->bitmap = NULL;
	}
	this->bitmap = new Bitmap(soubor);
	if (!bitmap)
	{
		FireViewChange();
		return E_FAIL;
	}
	else
	{
		if (bitmap->GetLastStatus() != Ok)
		{
			delete bitmap;
			bitmap = NULL;
			return E_FAIL;
		}
	}
	FireViewChange();
	return S_OK;
}

STDMETHOD(Ulozit)()
{
	if (!bitmap)
		return E_HANDLE;
	CLSID encoderCLSID;
	CString strPripona = CString(L"image/");
	if (wcscmp(this->pripona.m_str, L"jpg") == 0)
		strPripona.Append(L"jpeg");
	else
		strPripona.Append(this->pripona.m_str);
	if (GetKodekClsid(strPripona.GetBuffer(),
		&encoderCLSID) == -1)
		GetKodekClsid(L"image/jpeg", &encoderCLSID);
	Status status;
	status = bitmap->Save(this->soubor.m_str, &encoderCLSID);
	return (status == Ok) ? S_OK :S_FALSE;
}

static int GetKodekClsid(
	const wchar_t* format, CLSID* pClsid)
{
	UINT num = 0; // poet kodek
	UINT size = 0; // velikost pole kodek v bytech
	ImageCodecInfo* pImageCodecInfo = NULL;
	GetImageEncodersSize(&num, &size);
	if (size == 0)
		return -1;  // Chyba
	pImageCodecInfo = (ImageCodecInfo*)(malloc(size));
	if (!pImageCodecInfo)
		return -1;  // Chyba
	GetImageEncoders(num, size, pImageCodecInfo);
	for (UINT j = 0; j < num; ++j)
	{
		if (wcscmp(pImageCodecInfo[j].MimeType,
			format) == 0)
		{
			*pClsid = pImageCodecInfo[j].Clsid;
			free(pImageCodecInfo);
			return j;  // V podku
		}    
	}
	free(pImageCodecInfo);
	return -1; // Chyba
}

STDMETHOD(UlozitJakoBmp)(BSTR jmenoSouboru)
{
	if (!bitmap)
		return E_HANDLE;
	CLSID encoderCLSID;
	GetKodekClsid(L"image/bmp", &encoderCLSID);
	Status status = bitmap->Save(
		jmenoSouboru, &encoderCLSID);
	return (status == Ok) ? S_OK : S_FALSE;
}

STDMETHOD(UlozitJakoGif)(BSTR jmenoSouboru)
{
	if (!bitmap)
		return E_HANDLE;
	else
	{
		CLSID encoderCLSID;
		GetKodekClsid(L"image/gif", &encoderCLSID);
		return (bitmap->Save(
			jmenoSouboru, &encoderCLSID) == Ok)
			? S_OK :S_FALSE;
	}
}

STDMETHOD(UlozitJakoJpeg)(BSTR jmenoSouboru)
{
	if (!bitmap)
		return E_HANDLE;
	CLSID encoderCLSID;
	GetKodekClsid(L"image/jpeg", &encoderCLSID);
	Status status = bitmap->Save(jmenoSouboru, &encoderCLSID);
	return (status == Ok) ? S_OK :S_FALSE;
}

STDMETHOD(UlozitJakoJpeg2)(BSTR jmenoSouboru, ULONG kvalita)
{
	if (!bitmap)
		return E_HANDLE;
	CLSID encoderCLSID;
	GetKodekClsid(L"image/jpeg", &encoderCLSID);
	Status status;
	if (kvalita > 0 && kvalita <= 100)
	{
		EncoderParameters ePar;
		ePar.Count = 1;
		ePar.Parameter[0].Guid = EncoderQuality;
		ePar.Parameter[0].Type = EncoderParameterValueTypeLong;
		ePar.Parameter[0].NumberOfValues = 1;
		ePar.Parameter[0].Value = &kvalita;
		status = bitmap->Save(
			jmenoSouboru, &encoderCLSID, &ePar);
	}
	else
		status = bitmap->Save(
		jmenoSouboru, &encoderCLSID);
	return (status == Ok) ?
		S_OK :S_FALSE;
}

STDMETHOD(UlozitJakoPng)(BSTR jmenoSouboru)
{
	if (!bitmap)
		return E_HANDLE;
	else
	{
		CLSID encoderCLSID;
		GetKodekClsid(L"image/png", &encoderCLSID);
		return (bitmap->Save(jmenoSouboru,
			&encoderCLSID) == Ok) ? S_OK :S_FALSE;
	}
}

STDMETHOD(UlozitJakoTiff)(BSTR jmenoSouboru)
{
	if (!bitmap)
		return E_HANDLE;
	else
	{
		CLSID encoderCLSID;
		GetKodekClsid(L"image/tiff", &encoderCLSID);
		return (bitmap->Save(jmenoSouboru,
			&encoderCLSID) == Ok) ? S_OK :S_FALSE;
	}
}

STDMETHOD(InvertovatBarvy)()
{
	if (!this->bitmap)
		return E_HANDLE;
	BitmapData bData;
	this->bitmap->LockBits(&Rect(0, 0,
		bitmap->GetWidth(), bitmap->GetHeight()),
		ImageLockModeWrite | ImageLockModeRead,
		PixelFormat32bppARGB, &bData);
	BYTE* pData = (BYTE*)bData.Scan0;
	BYTE* pKonec = (BYTE*)((BYTE*)bData.Scan0 +
		(bData.Height * bData.Width * 4));
	while (pData < pKonec)
	{
		*pData = 0xFF - *pData;
		*(pData+1) = 0xFF - *(pData+1);
		*(pData+2) = 0xFF - *(pData+2);
		pData+=4;
	}
	bitmap->UnlockBits(&bData);
	FireViewChange();
	return S_OK;
}

STDMETHOD(PrevodNaStupneSede)()
{
	if (!this->bitmap)
		return E_HANDLE;
	BitmapData bData;
	this->bitmap->LockBits(&Rect(0, 0,
		bitmap->GetWidth(), bitmap->GetHeight()),
		ImageLockModeWrite | ImageLockModeRead,
		PixelFormat32bppARGB, &bData);
	BYTE* pData = (BYTE*)bData.Scan0;
	BYTE* pKonec = (BYTE*)((BYTE*)bData.Scan0 +
		(bData.Height * bData.Width * 4));
	BYTE b;
	while (pData < pKonec)
	{
		b = ((11* (*(pData+2)) + 59 *
		(*(pData+1))+ 30 * (*(pData)))) / 100;
		*(pData+3) = 255;
		memset(pData, b, 3);
		pData+=4;
	}
	bitmap->UnlockBits(&bData);
	FireViewChange();
	return S_OK;
}

STDMETHOD(ZmenitVelikost)(ULONG sirka,
	ULONG vyska, VARIANT_BOOL zachovatPomer)
{
	if (!this->bitmap)
		return E_HANDLE;
	if (zachovatPomer)
	{
		if ((float)sirka/(float)this->bitmap->GetWidth() >
			(float)vyska/(float)this->bitmap->GetHeight() )
			sirka = (this->bitmap->GetWidth()
			* vyska) / this->bitmap->GetHeight();
		else
			vyska = (this->bitmap->GetHeight() *
			sirka) / this->bitmap->GetWidth();
	}
	HDC hdcScreen = ::GetDC(NULL);
	if (!hdcScreen)
		throw 0;
	HDC hDC = ::CreateCompatibleDC(hdcScreen);
	if (!hDC)
		throw 0;
	HBITMAP hBitmap = ::CreateCompatibleBitmap(
		hdcScreen, sirka, vyska);
	if (!hBitmap)
		throw 0;
	::SelectObject(hDC, hBitmap);
	bool bVysl = KreslitBitmap(hDC, 0, 0,
		sirka, vyska, zachovatPomer != VARIANT_FALSE);
	if (!::DeleteDC(hDC))
		throw 0;
	if (!::ReleaseDC(NULL, hdcScreen))
		throw 0;
	if (bitmap)
	{
		delete bitmap;
		bitmap = NULL;
	}
	this->bitmap = new Bitmap(hBitmap, NULL);
	if ( !::DeleteObject(hBitmap) )
		throw 0;
	if (bVysl)
	{
		FireViewChange();
		return S_OK;
	}
	else
		return S_FALSE;
}

STDMETHOD(PreklopitSvisle)()
{
	if (!this->bitmap)
		return E_HANDLE;
	Status status;
	status = bitmap->RotateFlip(RotateNoneFlipY);
	FireViewChange();
	return (status == Ok) ?	S_OK : S_FALSE;
}

STDMETHOD(PreklopitVodorovne)()
{
	if (!this->bitmap)
		return E_HANDLE;
	Status status;
	status = bitmap->RotateFlip(RotateNoneFlipX);
	FireViewChange();
	return (status == Ok) ?	S_OK : S_FALSE;
}

STDMETHOD(OtocitVlevo90)()
{
	if (!this->bitmap)
		return E_HANDLE;
	Status status;
	status = bitmap->RotateFlip(Rotate270FlipNone);
	FireViewChange();
	return (status == Ok) ?	S_OK : S_FALSE;
}

STDMETHOD(OtocitVpravo90)()
{
	if (!this->bitmap)
		return E_HANDLE;
	Status status;
	status = bitmap->RotateFlip(Rotate90FlipNone);
	FireViewChange();
	return (status == Ok) ?	S_OK : S_FALSE;
}

STDMETHOD(NacistOpisObrazovky)()
{
	if (this->bitmap)
	{
		delete this->bitmap;
		this->bitmap = NULL;
	}
	int sirka = GetSystemMetrics(SM_CXSCREEN);
	int vyska = GetSystemMetrics(SM_CYSCREEN);
	HDC hdcScreen = ::GetDC(NULL);
	if (!hdcScreen)
		return E_HANDLE;
	HDC hdcBmp = CreateCompatibleDC(hdcScreen);
	if (!hdcBmp)
		return E_HANDLE;
	HBITMAP hBitmap = CreateCompatibleBitmap(
		hdcScreen, sirka, vyska);
	if (!hBitmap)
		return E_HANDLE;
	SelectObject(hdcBmp, hBitmap);
	BitBlt(hdcBmp, 0, 0, sirka, vyska,
		hdcScreen, 0, 0, SRCCOPY);
	// Vytvoit bitmapu
	this->bitmap = new Bitmap(hBitmap, NULL);

	if (hdcScreen)
		::ReleaseDC(NULL, hdcScreen);
	if (hdcBmp)
		::DeleteDC(hdcBmp);
	if (hBitmap)
		::DeleteObject(hBitmap);
	soubor = CComBSTR(L"obrazovka.jpg");
	pripona = CComBSTR(L"jpg");
	FireViewChange();
	if (this->bitmap)
		return (this->bitmap->GetLastStatus() == Ok)
		? S_OK : E_FAIL;
	else
		return E_FAIL;
}

STDMETHOD(Zavrit)()
{
	if (bitmap)
	{
		delete bitmap;
		bitmap = NULL;
	}
	return S_OK;
}

STDMETHOD(get_PruhlednePozadi)(VARIANT_BOOL* pHodnota)
{
	*pHodnota = (VARIANT_BOOL)pruhlednePozadi;
	return S_OK;
}

STDMETHOD(put_PruhlednePozadi)(VARIANT_BOOL hodnota)
{
	pruhlednePozadi = (hodnota != VARIANT_FALSE);
	FireViewChange();
	return S_OK;
}


STDMETHOD(VybratSoubor)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter = L"Grafick soubory\0"
		L"*.bmp;*.jpg;*.tiff;"
		L"*.png;*.gif;*.jpeg\0"
		L"Vechny soubory\0"
		L"*.*\0";
	ofn.nFilterIndex = 1;
	ofn.Flags = OFN_FILEMUSTEXIST;
	if (!GetOpenFileName(&ofn))
		return S_FALSE;
	return Nacist(szSoubor);
}

STDMETHOD(UlozitVybratSouborJpeg)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter = L"JPEG (*.jpg)\0*.jpg\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"jpg";
	if (!GetSaveFileName(&ofn))
		return false;
	soubor = CComBSTR(szSoubor);
	UlozitJakoJpeg(szSoubor);
	return true;
}

STDMETHOD(UlozitVybratSouborBmp)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter =
		L"Bitmapy (*.bmp)\0*.bmp\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"bmp";
	if (!GetSaveFileName(&ofn))
		return false;
	soubor = CComBSTR(szSoubor);
	UlozitJakoBmp(szSoubor);
	return true;
}

STDMETHOD(UlozitVybratSouborGif)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter =
		L"GIF (*.gif)\0*.gif\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"gif";
	if (!GetSaveFileName(&ofn))
		return false;
	soubor = CComBSTR(szSoubor);
	UlozitJakoGif(szSoubor);
	return true;
}

STDMETHOD(UlozitVybratSouborPng)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter =
		L"PNG (*.png)\0*.png\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"png";
	if (!GetSaveFileName(&ofn))
		return false;
	soubor = CComBSTR(szSoubor);
	UlozitJakoPng(szSoubor);
	return true;
}

STDMETHOD(UlozitVybratSouborTiff)()
{
	OPENFILENAME ofn;
	wchar_t szSoubor[MAX_PATH] = {0};
	if (this->soubor.Length() > 0)
		if (PathFileExistsW(soubor.m_str))
	        wcsncpy(szSoubor,
			this->soubor.m_str, MAX_PATH);
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.lpstrFile = szSoubor;
	ofn.nMaxFile = MAX_PATH;
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFilter =
		L"TIFF (*.tif)\0*.tif\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"tif";
	if (!GetSaveFileName(&ofn))
		return false;
	soubor = CComBSTR(szSoubor);
	UlozitJakoTiff(szSoubor);
	return true;
}
};

OBJECT_ENTRY_AUTO(__uuidof(ImageAx), ImageAx)
