/****************************************************
Seznam natench modul procesu
Soubor: SeznamModulu.h
(C) 2005 Radek Chalupa - www.radekchalupa.cz
*****************************************************/

#pragma once

#include "resource.h"
#include "ModulProcesy.h"

// SeznamModulu
class ATL_NO_VTABLE SeznamModulu : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<SeznamModulu, &CLSID_SeznamModulu>,
	public CWindowImpl< SeznamModulu >,
	public AtlDemo::Aplikace,
	public IDispatchImpl<ISeznamModulu,
		&IID_ISeznamModulu,
		&LIBID_ModulProcesyLib, 1, 0>
{
public:
	static CWndClassInfo& GetWndClassInfo()
	{
		static CWndClassInfo wc =
		{
			{
				sizeof(WNDCLASSEX),
				CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW,
				StartWindowProc,
				0, 0,
				NULL,
				(HICON)NULL,
				(HCURSOR)NULL,
				(HBRUSH)GetSysColorBrush(COLOR_WINDOW),
				NULL, // menu
				L"SeznamModuluAplikace",
				(HICON)NULL 
			},
			NULL, // origname
			NULL, // wndproc
			IDC_ARROW, // cursor ID
			TRUE, // system cursor
			0, // atom
			L""
		};
		return wc;
	}

	static unsigned __stdcall ThreadProc(void* parametr)
	{
		try
		{
			if (!parametr)
				return -1;
			CoInitialize(NULL);
			HRESULT hresult;
			((SeznamModulu*)parametr)->VytvoritOkno();
			MSG msg;
			while (GetMessage(&msg, 0, 0, 0) > 0)
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			if (((SeznamModulu*)parametr)->hThread)
			{
				if (!CloseHandle(
					((SeznamModulu*)parametr)->hThread))
					ExitProcess(-1);
				((SeznamModulu*)parametr)->hThread = NULL;
			}
			CoUninitialize();
			return 0;
		}
		catch (...)
		{
			CoUninitialize();
			return -1;
		}
	}
public:
	HANDLE hThread;
	HWND hListBox;
	DWORD intervalAktualizace;
	bool timerSpusten;
	CString strAplikace;
	HICON hIkona;
public:
	SeznamModulu()
	{
		hIkona = NULL;
		strAplikace = L"Seznam modul";
		hThread = NULL;
		intervalAktualizace = 3000;
		timerSpusten = false;
	}

	DECLARE_REGISTRY_RESOURCEID(IDR_SEZNAMMODULU)
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	BEGIN_COM_MAP(SeznamModulu)
		COM_INTERFACE_ENTRY(ISeznamModulu)
		COM_INTERFACE_ENTRY(IDispatch)
	END_COM_MAP()

	BEGIN_MSG_MAP(Aplikace)
		MESSAGE_HANDLER(WM_CLOSE, OnClose)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
		MESSAGE_HANDLER(WM_TIMER, OnTimer)
		REFLECT_NOTIFICATIONS()
	END_MSG_MAP()

	LRESULT OnClose(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		if (timerSpusten)
			if (!KillTimer(1))
				FatalniChyba();
		return DefWindowProc();
	}

	LRESULT OnDestroy(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		PostQuitMessage(0);
		return DefWindowProc();
	}

	LRESULT OnTimer(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		if (wParam == 1)
			Aktualizovat();
		return DefWindowProc();
	}

	LRESULT OnSize(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		if (::IsWindow(hListBox))
			::SetWindowPos(hListBox,
			NULL, 0, 0,
			LOWORD(lParam),
			HIWORD(lParam),
			SWP_NOZORDER);
		return DefWindowProc();
	}

	LRESULT OnSysCommand(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		switch (wParam)
		{
		case SC_CLOSE:
			PostMessage(WM_SYSCOMMAND, SC_MINIMIZE);
			bHandled = TRUE;
			return 0;
		}
		return DefWindowProc();
	}

	void VytvoritOkno()
	{
		RECT rect = {0, 0, 794, 570};
		if (!Create(NULL, &rect, strAplikace.GetBuffer(),
			WS_OVERLAPPEDWINDOW, 0))
			FatalniChyba();
		hListBox = CreateWindowEx(0,
			WC_LISTBOX, NULL,
			WS_CHILD | WS_VISIBLE |
			WS_VSCROLL | WS_HSCROLL |
			LBS_NOINTEGRALHEIGHT | 
			LBS_SORT,
			0, 0,
			0, 0,
			m_hWnd,
			NULL,
			_AtlBaseModule.m_hInst,
			NULL);
		::SendMessage(hListBox, WM_SETFONT,
			(WPARAM)GetStockObject(DEFAULT_GUI_FONT),
			TRUE);
		if (!SetTimer(1, intervalAktualizace))
			FatalniChyba();
		timerSpusten = true;
		if (hIkona)
		{
			SetIcon(hIkona, FALSE);
			SetIcon(hIkona, TRUE);
		}
		CenterWindow();
		ShowWindow(SW_SHOW);
		Aktualizovat();
	}
	
	void FatalniChyba()
	{
		ZobrazitPosledniChybu();
		ExitProcess(-1);
	}

	bool Aktualizovat()
	{
		if (!::IsWindow(hListBox))
			return false;
		::SendMessage(hListBox, LB_RESETCONTENT, 0, 0);
		HMODULE hModuly[1024];
		DWORD velikost;
		HANDLE hProces;
		hProces = OpenProcess(PROCESS_QUERY_INFORMATION |
			PROCESS_VM_READ, FALSE, GetCurrentProcessId());
		if (!hProces)
			return false;
		if (EnumProcessModules(hProces,
			hModuly, sizeof(hModuly), &velikost))
		{
			for (int i = 0;
				i < (velikost/sizeof(HMODULE));
				i++)
			{
				wchar_t soubor[MAX_PATH];
				if (GetModuleFileNameEx(hProces,
					hModuly[i], soubor, sizeof(soubor)))
					::SendMessage(hListBox, LB_ADDSTRING,
					0, (LPARAM)soubor);
			}
		}
		RedrawWindow();
	}

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
		if (IsWindow())
			PostMessage(WM_CLOSE);
		Sleep(250);
	}

public:
	STDMETHOD(Spustit)(BSTR jmenoAplikace,
		OLE_HANDLE ikonaAplikace)
	{
		if (hThread)
			return S_FALSE;
		if (jmenoAplikace)
			this->strAplikace = CString(jmenoAplikace);
		if (ikonaAplikace)
			this->hIkona = (HICON)ikonaAplikace;
		unsigned int ui;
		this->hThread = (HANDLE)_beginthreadex(NULL, 0,
			SeznamModulu::ThreadProc, (void*)this, 0, &ui);
		if (!this->hThread)
			return E_FAIL;
		else
			return S_OK;
	}
	STDMETHOD(get_IntervalAktualizace)(ULONG* pInterval)
	{
		*pInterval = intervalAktualizace;
		return S_OK;
	}

	STDMETHOD(put_IntervalAktualizace)(ULONG interval)
	{
		if (interval < 100)
			return E_INVALIDARG;
		if (timerSpusten)
			if (!KillTimer(1))
				FatalniChyba();
		if (IsWindow())
		{
			if (!SetTimer(1, interval))
				FatalniChyba();
			timerSpusten = true;
		}
		return S_OK;
	}
};

OBJECT_ENTRY_AUTO(__uuidof(SeznamModulu), SeznamModulu)
