// Aplikace.h : Declaration of the OknoPlus

#pragma once

#include "resource.h"
#include "OknoPlus.h"


// OknoPlus

class ATL_NO_VTABLE OknoPlus : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<OknoPlus, &CLSID_OknoPlus>,
	public IDispatchImpl<IOknoPlus, &IID_IOknoPlus,
		&LIBID_OknoPlusLib, 1, 0>,
		public CWindowImpl< OknoPlus >
{
public:
	static const UINT wmOznamovaciIkona = (WM_APP + 2000);

	static const int vlevoNahore = 1;
	static const int vlevoDole = 2;
	static const int vpravoNahore = 3;
	static const int vpravoDole = 4;
	static const int centrovat = 5;
	static const int bezZarovnani = 6;
	static const int polovinaVlevo = 7;
	static const int polovinaVpravo = 8;
	static const int polovinaNahore = 9;
	static const int polovinaDole = 10;
	static const int navrchu = 11;
	static const int pruhlednostAlfa = 12;
	static const int pruhlednostBarva = 13;
	static const int pruhlednostOboji = 14;
	static const int pruhlednostZadna = 15;
public:
	bool minimDoIkony;
	bool tazeniMysi;
	bool pridatPolSysMenu;
	bool zavritNaEscape;
	bool ikonaExistuje;
	UINT idIkona;
	HMENU hmenuPopup;
	ULONG vychoziProcPruhl;
	COLORREF crBarvaOkna;

public:
	OknoPlus()
	{
		crBarvaOkna = GetSysColor(COLOR_WINDOW);
		idIkona = 100;
		ikonaExistuje = false;
		hmenuPopup = NULL;
		minimDoIkony = false;
		pridatPolSysMenu = true;
		tazeniMysi = true;
		vychoziProcPruhl = 50;
		zavritNaEscape = false;
	}
	
	DECLARE_REGISTRY_RESOURCEID(IDR_OKNOPLUS1)
	DECLARE_PROTECT_FINAL_CONSTRUCT()

	BEGIN_COM_MAP(OknoPlus)
		COM_INTERFACE_ENTRY(IOknoPlus)
		COM_INTERFACE_ENTRY(IDispatch)
	END_COM_MAP()

	BEGIN_MSG_MAP(OknoPlus)
		MESSAGE_HANDLER(WM_CLOSE, OnClose)
		MESSAGE_HANDLER(WM_KEYUP, OnKeyUp)
		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
		MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
		MESSAGE_HANDLER(wmOznamovaciIkona, OnOznamovaciIkona)
	END_MSG_MAP()

	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if (tazeniMysi)
		{
			SendMessage(WM_SYSCOMMAND, SC_MOVE | HTCAPTION);
			bHandled = TRUE;
			return 0;
		}
		return DefWindowProc();
	}

	LRESULT OnKeyUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		switch (wParam)
		{
		case VK_ESCAPE:
			if (zavritNaEscape)
				PostMessage(WM_CLOSE);
			break;
		}

		return DefWindowProc();
	}

	LRESULT OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		ZrusitOznamovaciIkonu();
		return DefWindowProc();
	}

	LRESULT OnSysCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		switch (wParam)
		{
		case SC_MINIMIZE:
			VytvoritOznamovaciIkonu();
			break;
		case SC_RESTORE:
			ZrusitOznamovaciIkonu();
			break;
		case ID_OKNO_CENTROVAT:
			Zarovnat(centrovat);
			break;
		case ID_OKNO_NAVRCHU:
			PrepnoutNavrchu();
			if (GetNavrchu())
				CheckMenuItem(GetSystemMenu(FALSE),
					ID_OKNO_NAVRCHU, MF_CHECKED | MF_BYCOMMAND);
			else
				CheckMenuItem(GetSystemMenu(FALSE),
					ID_OKNO_NAVRCHU, MF_UNCHECKED | MF_BYCOMMAND);

			break;
		case ID_OKNO_POLOVINA_DOLE:
			Zarovnat(polovinaDole);
			break;
		case ID_OKNO_POLOVINA_NAHORE:
			Zarovnat(polovinaNahore);
			break;
		case ID_OKNO_POLOVINA_VLEVO:
			Zarovnat(polovinaVlevo);
			break;
		case ID_OKNO_POLOVINA_VPRAVO:
			Zarovnat(polovinaVpravo);
			break;
		case ID_OKNO_VLEVO_DOLE:
			Zarovnat(vlevoDole);
			break;
		case ID_OKNO_VLEVO_NAHORE:
			Zarovnat(vlevoNahore);
			break;
		case ID_OKNO_VPRAVO_DOLE:
			Zarovnat(vpravoDole);
			break;
		case ID_OKNO_VPRAVO_NAHORE:
			Zarovnat(vpravoNahore);
			break;
		case ID_PRUHLEDNOST_ALFA:
			SetProcentoPruhlednosti(vychoziProcPruhl);
			break;
		case ID_PRUHLEDNOST_BARVA:
			SetPruhlednaBarva(crBarvaOkna);
			break;
		case ID_PRUHLEDNOST_OBOJI:
			SetPruhlednost(crBarvaOkna,
				vychoziProcPruhl);
			break;
		case ID_ZRUSIT_PRUHLEDNOST:
			SetProcentoPruhlednosti(0);
			break;
		case ID_SKRYT_OKNO:
			VytvoritOznamovaciIkonu();
			ShowWindow(SW_HIDE);
			break;
		}
		return DefWindowProc();
	}

	LRESULT OnOznamovaciIkona(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if (wParam != idIkona)
			return 0;
		switch (lParam)
		{
		case WM_LBUTTONUP:
			ZrusitOznamovaciIkonu();
			ShowWindow(SW_RESTORE);
			SetForegroundWindow(m_hWnd);
			break;
		}
		return 0;
	}

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
        ZrusitOznamovaciIkonu();
		Uvolnit();
	}

public:

	STDMETHOD(Nastavit)(OLE_HANDLE hOkno)
	{
		if (!::IsWindow((HWND)hOkno))
			return E_INVALIDARG;
		if (IsWindow())
			UnsubclassWindow();
		if (!SubclassWindow((HWND)hOkno))
			return E_FAIL;
		if (pridatPolSysMenu)
			PridatPolozkySystemMenu();
		HBRUSH hb = (HBRUSH)GetClassLongPtr(m_hWnd,
			GCLP_HBRBACKGROUND);
		if (!hb)
			return S_OK;
		LOGBRUSH lb;
		GetObject(hb, sizeof(lb), &lb);
		if (lb.lbStyle == BS_SOLID)
			crBarvaOkna = lb.lbColor;
		return S_OK;
	}

	STDMETHOD(Uvolnit)()
	{
		ZrusitOznamovaciIkonu();
		if (IsWindow())
			UnsubclassWindow();
		return S_OK;
	}

	STDMETHOD(get_TazeniMysi)(VARIANT_BOOL* pVal)
	{
		*pVal = (VARIANT_BOOL)this->tazeniMysi;
		return S_OK;
	}

	STDMETHOD(put_TazeniMysi)(VARIANT_BOOL newVal)
	{
		this->tazeniMysi = newVal != VARIANT_FALSE;
		return S_OK;
	}

	STDMETHOD(get_ZavritNaEscape)(VARIANT_BOOL* pVal)
	{
		*pVal = (VARIANT_BOOL)this->zavritNaEscape;
		return S_OK;
	}

	STDMETHOD(put_ZavritNaEscape)(VARIANT_BOOL newVal)
	{
		this->zavritNaEscape = newVal != VARIANT_FALSE;
		return S_OK;
	}

	STDMETHOD(get_MinimalizovatDoTrayIkony)(VARIANT_BOOL* pVal)
	{
		*pVal = this->minimDoIkony;
		return S_OK;
	}

	STDMETHOD(put_MinimalizovatDoTrayIkony)(VARIANT_BOOL newVal)
	{
		minimDoIkony = newVal != VARIANT_FALSE;
		return S_OK;
	}

	STDMETHOD(VytvoritOznamovaciIkonu)()
	{
		if (ikonaExistuje)
			return S_OK;
		if (!IsWindow())
			return E_HANDLE;
		NOTIFYICONDATA nid;
		CString text;
		memset(&nid, 0, sizeof(NOTIFYICONDATA));
		nid.cbSize = sizeof(NOTIFYICONDATA);
		if (!GetWindowText(nid.szTip, sizeof(nid.szTip) / sizeof(wchar_t)) > 0)
			wcscpy(nid.szTip, L"Okno Plus");
		nid.hIcon = (HICON)SendMessage(WM_GETICON, ICON_SMALL);
		if (!nid.hIcon)
			nid.hIcon = (HICON)::GetClassLongPtr(m_hWnd, GCLP_HICONSM);
		if (!nid.hIcon)
			nid.hIcon = (HICON)SendMessage(WM_GETICON, ICON_BIG);
		if (!nid.hIcon)
			nid.hIcon = (HICON)::GetClassLongPtr(m_hWnd, GCLP_HICON);
		if (!nid.hIcon)
			nid.hIcon = LoadIcon(NULL, IDI_APPLICATION);
		nid.hWnd = m_hWnd;
		nid.uCallbackMessage = wmOznamovaciIkona;
		nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
		nid.uID = this->idIkona;
		if (!Shell_NotifyIcon(NIM_ADD, &nid))
			return E_FAIL;
		ikonaExistuje = true;
		return S_OK;
	}

	STDMETHOD(ZrusitOznamovaciIkonu)()
	{
		if (!ikonaExistuje)
			return S_OK;
		if (!IsWindow())
			return E_HANDLE;
		NOTIFYICONDATA nid;
		ZeroMemory(&nid, sizeof(nid));
		nid.cbSize = sizeof(NOTIFYICONDATA);
		nid.hWnd = m_hWnd;
		nid.uID = this->idIkona;
		if (!Shell_NotifyIcon(NIM_DELETE, &nid))
			return E_FAIL;
		ikonaExistuje = false;
		return S_OK;
	}

	STDMETHOD(get_PolozkySysNabidky)(VARIANT_BOOL* pVal)
	{
		*pVal = (VARIANT_BOOL)
			this->pridatPolSysMenu;
		return S_OK;
	}

	STDMETHOD(put_PolozkySysNabidky)(VARIANT_BOOL newVal)
	{
		this->pridatPolSysMenu =
			newVal != VARIANT_FALSE;
		return S_OK;
	}

	STDMETHOD(PridatPolozkySystemMenu)()
	{
		if (hmenuPopup) // u byly pidny
			return S_OK;
		HMENU hmenu = GetSystemMenu(FALSE);
		if (!hmenu)
			return E_HANDLE;
		this->hmenuPopup = LoadMenu(_AtlBaseModule.m_hInst,
			MAKEINTRESOURCE(IDR_POPUP_OKNO));
		if (!this->hmenuPopup)
			return E_HANDLE;
		PridatPolozkuMenu(hmenu, ID_OKNO_NAVRCHU, SC_RESTORE, L"Navrchu", true);
		PridatPolozkuMenu(hmenu, ID_SKRYT_OKNO, SC_RESTORE, L"Skrt do ikony", true);

		MENUITEMINFO mii;
		ZeroMemory(&mii, sizeof(mii));
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_STRING | MIIM_DATA | MIIM_SUBMENU; 
		mii.fType = MFT_STRING;
		mii.hSubMenu = GetSubMenu(this->hmenuPopup, 0);
		mii.dwTypeData = L"Dal monosti";
		if (!InsertMenuItem(hmenu, SC_RESTORE, FALSE, &mii))
			return E_FAIL;

		PridatOddelovacMenu(hmenu, SC_RESTORE, ID_ODDELOVAC_SYSTEMMENU);
		return S_OK;
	}

	bool PridatPolozkuMenu(HMENU hmenu, UINT uId, UINT uIdPred,
		wchar_t* text, bool prepinac = false)
	{
		if (!hmenu)
			return false;
		MENUITEMINFO mii;
		memset(&mii, 0, sizeof(MENUITEMINFO));
		mii.cbSize = sizeof(MENUITEMINFO);
		mii.fMask = MIIM_STATE | MIIM_ID | MIIM_STRING | MIIM_FTYPE;
		mii.fState = MFS_ENABLED;
		mii.fState = MFS_ENABLED;
		mii.dwTypeData = text;
		mii.wID = uId;
		return InsertMenuItem(hmenu, uIdPred, FALSE, &mii) != FALSE;
	}

	bool PridatOddelovacMenu(HMENU hmenu, UINT uIdPred, UINT uId = 0)
	{
		if (!hmenu)
			return E_INVALIDARG;
		MENUITEMINFO mii;
		ZeroMemory(&mii, sizeof(MENUITEMINFO));
		mii.cbSize = sizeof(MENUITEMINFO);
		mii.fMask = MIIM_STATE | MIIM_ID | MIIM_FTYPE;
		mii.fType = MFT_SEPARATOR;
		mii.fState = MFS_DISABLED;
		mii.dwTypeData = 0;
		mii.wID = uId;
		return InsertMenuItem(hmenu, uIdPred, FALSE, &mii) != FALSE;
	}

	STDMETHOD(OdebratPolozkySystemMenu)()
	{
		if (!hmenuPopup) // nebylo pidno
			return S_OK;
		HMENU hmenu = GetSystemMenu(FALSE);
		if (!hmenu)
			return E_HANDLE;
		if (this->hmenuPopup)
		{
			RemoveMenu(hmenu, (UINT)2, MF_BYPOSITION);
			if (!DestroyMenu(hmenuPopup))
				return E_FAIL;
			hmenuPopup = NULL;
		}
		RemoveMenu(hmenu, ID_OKNO_NAVRCHU, MF_BYCOMMAND);
		RemoveMenu(hmenu, ID_SKRYT_OKNO, MF_BYCOMMAND);
		RemoveMenu(hmenu, ID_ODDELOVAC_SYSTEMMENU, MF_BYCOMMAND);
		return S_OK;
	}

	void Zarovnat(DWORD zarovnani)
	{
		if (!IsWindow())
			AtlThrow(E_INVALIDARG);
		if (zarovnani == bezZarovnani)
			return;
		RECT rectOkno;
		RECT rectWork;
		if (!GetWindowRect(&rectOkno))
			throw 0;
		int sirka = rectOkno.right - rectOkno.left;
		int vyska = rectOkno.bottom - rectOkno.top;
		int xPos = 0;
		int yPos = 0;
		if (!SystemParametersInfo(SPI_GETWORKAREA, 0, &rectWork, 0))
			throw 0;
		switch (zarovnani)
		{
		case vlevoNahore:
			xPos = rectWork.left;
			yPos = rectWork.top;
			SetWindowPos(NULL, xPos, yPos, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
			break;
		case vpravoNahore:
			xPos = rectWork.right - sirka;
			yPos = rectWork.top;
			SetWindowPos(NULL, xPos, yPos, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
			break;
		case vlevoDole:
			xPos = rectWork.left;
			yPos = rectWork.bottom - vyska;
			SetWindowPos(NULL, xPos, yPos, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
			break;
		case vpravoDole:
			xPos = rectWork.right - sirka;
			yPos = rectWork.bottom - vyska;
			SetWindowPos(NULL, xPos, yPos, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
			break;
		case polovinaDole:
			xPos = 0;
			yPos = (rectWork.bottom - rectWork.top) / 2;
			if (GetWindowLongPtr(GWL_STYLE) & WS_SIZEBOX)
			{
				vyska = (rectWork.bottom - rectWork.top) / 2;
				sirka = rectWork.right - rectWork.left;
			}
			SetWindowPos(NULL, xPos, yPos, sirka, vyska, SWP_NOZORDER);
			break;
		case polovinaNahore:
			xPos = 0;
			yPos = 0;
			if (GetWindowLongPtr(GWL_STYLE) & WS_SIZEBOX)
			{
				vyska = (rectWork.bottom - rectWork.top) / 2;
				sirka = rectWork.right - rectWork.left;
			}
			SetWindowPos(NULL, xPos, yPos, sirka, vyska, SWP_NOZORDER);
			break;
		case polovinaVlevo:
			xPos = 0;
			yPos = 0;
			if (GetWindowLongPtrW(GWL_STYLE) & WS_SIZEBOX)
			{
				vyska = rectWork.bottom - rectWork.top;
				sirka = (rectWork.right - rectWork.left) / 2;
			}
			SetWindowPos(NULL, xPos, yPos, sirka, vyska, SWP_NOZORDER);
			break;
		case polovinaVpravo:
			xPos = (rectWork.right - rectWork.left) / 2;
			yPos = 0;
			if ( GetWindowLongPtr(GWL_STYLE) & WS_SIZEBOX )
			{
				vyska = rectWork.bottom - rectWork.top;
				sirka = (rectWork.right - rectWork.left)/ 2;
			}
			SetWindowPos(NULL, xPos, yPos, sirka, vyska, SWP_NOZORDER);
			break;
		case centrovat:
			CenterWindow(GetDesktopWindow());
			break;
		}
	}

	void PrepnoutNavrchu()
	{
		if (!IsWindow())
			return;
		if ((DWORD)GetWindowLongPtr(
			GWL_EXSTYLE) & WS_EX_TOPMOST)
			SetWindowPos(HWND_NOTOPMOST,
			0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		else
			SetWindowPos(HWND_TOPMOST,
			0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}

	bool GetNavrchu()
	{
		return ((GetWindowLongPtr(GWL_EXSTYLE) & WS_EX_TOPMOST) != 0);
	}

	// Nastav procento prhlednosti (alfa hodnotou)
	void SetProcentoPruhlednosti(int iProcento)
	{
		if ( iProcento < 0 || iProcento > 100 )
			AtlThrow(E_INVALIDARG);
		LONG_PTR lStyle;
		lStyle = GetWindowLongPtr(GWL_EXSTYLE);
		if ( iProcento > 0 )
		{
			SetWindowLongPtr(GWL_EXSTYLE, lStyle | WS_EX_LAYERED);
			SetLayeredWindowAttributes(m_hWnd, 0, ((100-iProcento) * 255) / 100, LWA_ALPHA);
		}
		else
		{
			lStyle &=~ WS_EX_LAYERED;
			SetWindowLongPtr(GWL_EXSTYLE, lStyle);
		}
	}

	// Nastav procento prhlednosti (kdem barvy)
	void SetPruhlednaBarva(COLORREF crBarva)
	{
		if ( crBarva & 0xFF000000 )
			AtlThrow(E_INVALIDARG);
		LONG_PTR lStyle;
		lStyle = GetWindowLongPtr(GWL_EXSTYLE);
		SetWindowLongPtr(GWL_EXSTYLE, lStyle | WS_EX_LAYERED);
		SetLayeredWindowAttributes(m_hWnd, crBarva, 0, LWA_COLORKEY);
	}

	//
	// Nastav prhlednosti kdem barvy i alfa hodnotou souasn
	//
	void SetPruhlednost(COLORREF crBarva, int nProcento)
	{
		if (crBarva & 0xFF000000)
			AtlThrow(E_INVALIDARG);
		if (nProcento < 1 || nProcento > 100)
			AtlThrow(E_INVALIDARG);
		LONG_PTR lStyle;
		lStyle = GetWindowLongPtr(GWL_EXSTYLE);
		SetWindowLongPtr(GWL_EXSTYLE, lStyle | WS_EX_LAYERED);
		SetLayeredWindowAttributes(m_hWnd, crBarva, ((100 - nProcento) * 255) / 100,
			LWA_COLORKEY | LWA_ALPHA);
	}

	STDMETHOD(get_VychoziProcentoPruhlednosti)(ULONG* pVal)
	{
		*pVal = vychoziProcPruhl;
		return S_OK;
	}

	STDMETHOD(put_VychoziProcentoPruhlednosti)(ULONG newVal)
	{
		vychoziProcPruhl = newVal;
		return S_OK;
	}
};

OBJECT_ENTRY_AUTO(__uuidof(OknoPlus), OknoPlus)
