/****************************************************
MediaplayerAx - ActiveX
Soubor: MediaplayerAx.h
(C) 2005 Radek Chalupa - www.radekchalupa.cz
*****************************************************/

#pragma once

#include "resource.h"
#include <atlctl.h>
#include "PrehravacAx.h"
#include "_IMediaplayerAxEvents_CP.h"

// MediaplayerAx
class ATL_NO_VTABLE MediaplayerAx : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CStockPropImpl<MediaplayerAx, IMediaplayerAx>,
	public IPersistStreamInitImpl<MediaplayerAx>,
	public IOleControlImpl<MediaplayerAx>,
	public IOleObjectImpl<MediaplayerAx>,
	public IOleInPlaceActiveObjectImpl<MediaplayerAx>,
	public IViewObjectExImpl<MediaplayerAx>,
	public IOleInPlaceObjectWindowlessImpl<MediaplayerAx>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<MediaplayerAx>,
	public CProxy_IMediaplayerAxEvents<MediaplayerAx>, 
	public IPersistStorageImpl<MediaplayerAx>,
	public ISpecifyPropertyPagesImpl<MediaplayerAx>,
	public IQuickActivateImpl<MediaplayerAx>,
	public IDataObjectImpl<MediaplayerAx>,
	public IProvideClassInfo2Impl<&CLSID_MediaplayerAx,
		&__uuidof(_IMediaplayerAxEvents), &LIBID_PrehravacAxLib>,
	public CComCoClass<MediaplayerAx, &CLSID_MediaplayerAx>,
	public CComControl<MediaplayerAx>
{
public:
	CComPtr< IGraphBuilder > graphBuilder;
	IMediaControl* mediaControl;
	IMediaEvent* mediaEvent;
	IVideoWindow* videoWindow;
	IBasicVideo* basicVideo;
	IBasicAudio* basicAudio;
	HRESULT hresult;
	CComBSTR soubor;
	bool obsahujeVideo;
	bool tazeniMysi;

public:
	MediaplayerAx()
	{
		hresult = S_OK;
		mediaControl = NULL;
		mediaEvent = NULL;
		videoWindow = NULL;
		basicAudio = NULL;
		basicVideo = NULL;
		graphBuilder = NULL;
		obsahujeVideo = false;
		tazeniMysi = true;
		m_bWindowOnly = TRUE;
	}

DECLARE_OLEMISC_STATUS(OLEMISC_RECOMPOSEONRESIZE | 
	OLEMISC_CANTLINKINSIDE | 
	OLEMISC_INSIDEOUT | 
	OLEMISC_ACTIVATEWHENVISIBLE | 
	OLEMISC_SETCLIENTSITEFIRST
)

DECLARE_REGISTRY_RESOURCEID(IDR_MEDIAPLAYERAX)

BEGIN_COM_MAP(MediaplayerAx)
	COM_INTERFACE_ENTRY(IMediaplayerAx)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow,
		IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(MediaplayerAx)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
END_PROP_MAP()

BEGIN_CONNECTION_POINT_MAP(MediaplayerAx)
	CONNECTION_POINT_ENTRY(__uuidof(_IMediaplayerAxEvents))
END_CONNECTION_POINT_MAP()

BEGIN_MSG_MAP(MediaplayerAx)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
	CHAIN_MSG_MAP(CComControl<MediaplayerAx>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IMediaplayerAx,
		};

		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND |
	VIEWSTATUS_OPAQUE)

// IMediaplayerAx
public:
	HRESULT OnDraw(ATL_DRAWINFO& di)
	{
		RECT& rc = *(RECT*)di.prcBounds;
		HRGN hRgnOld = NULL;
		if (GetClipRgn(di.hdcDraw, hRgnOld) != 1)
			hRgnOld = NULL;
		bool bSelectOldRgn = false;
		HRGN hRgnNew = CreateRectRgn(
			rc.left, rc.top, rc.right, rc.bottom);
		if (hRgnNew != NULL)
			bSelectOldRgn =
			(SelectClipRgn(di.hdcDraw, hRgnNew) != ERROR);

		Rectangle(di.hdcDraw,
			rc.left, rc.top, rc.right, rc.bottom);
		SetTextAlign(di.hdcDraw, TA_CENTER|TA_BASELINE);
		LPCTSTR pszText = L"Mdiapehrva";
		TextOut(di.hdcDraw, 
			(rc.left + rc.right) / 2, 
			(rc.top + rc.bottom) / 2, 
			pszText, wcslen(pszText));
		if (bSelectOldRgn)
			SelectClipRgn(di.hdcDraw, hRgnOld);
		if (hRgnNew)
			if (!DeleteObject(hRgnNew))
				return E_FAIL;
		return S_OK;
	}

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
	}

	STDMETHOD(Otevrit)(BSTR soubor)
	{
		if (!graphBuilder)
			if (VytvoritGraf() != S_OK)
				return E_FAIL;
		try
		{
			hresult = graphBuilder->RenderFile(
				soubor, NULL);
			if (FAILED(hresult))
				throw 0;
			
			hresult = graphBuilder->QueryInterface(
				&videoWindow);
			if (FAILED(hresult))
				throw 0;
			hresult = graphBuilder->QueryInterface(
				&basicVideo);
			if (FAILED(hresult))
				throw 0;
			hresult = graphBuilder->QueryInterface(
				&basicAudio);
			if (FAILED(hresult))
				throw 0;
			LONG sirka;
			videoWindow->get_Width(&sirka);
			obsahujeVideo = (sirka > 0);

			if (obsahujeVideo)
			{
				hresult = videoWindow->put_WindowStyle(
					WS_POPUP | WS_CHILD
					| WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
				if (FAILED(hresult))
					throw 0;
				hresult = videoWindow->put_Left(0);
				if (FAILED(hresult))
					throw 0;
				hresult = videoWindow->put_Top(0);
				if (FAILED(hresult))
					throw 0;
				hresult = videoWindow->put_Owner(
					(OAHWND)m_hWnd);
				if (FAILED(hresult))
					throw 0;
				hresult = videoWindow->put_Visible(OATRUE);
				if (FAILED(hresult))
					throw 0;
				hresult = videoWindow->put_MessageDrain(
					(OAHWND)m_hWnd);	
				if (FAILED(hresult))
					throw 0;
			}
			hresult = mediaControl->Run();
			if (FAILED(hresult))
				throw 0;
			Fire_Spusteno();
			if (obsahujeVideo)
			{
				LONG sirka;
				LONG vyska;
				videoWindow->get_Width(&sirka);
				videoWindow->get_Height(&vyska);
				Fire_ZmenaRozmeruVidea(sirka, vyska);
			}
			else
				Fire_ZmenaRozmeruVidea(450, 300);
			return S_OK;
		}
		catch (...)
		{
			Fire_Chyba(hresult,
				L"Nepodailo se otevt soubor");
			return E_FAIL;
		}
		return S_OK;
	}

	HRESULT VytvoritGraf()
	{
		HRESULT hr;
		if (graphBuilder)
			return S_OK;
		hr = graphBuilder.CoCreateInstance(
			CLSID_FilterGraph);
		if (hr != S_OK)
			return hr;
		hr = graphBuilder->QueryInterface(
			&mediaControl);
		if (FAILED(hr))
			return hr;
		hr = graphBuilder->QueryInterface(
			&mediaEvent);
		if (FAILED(hr))
			return hr;
		return S_OK;
	}

	void ZrusitGraf()
	{
		if (!graphBuilder)
			return;
		if (mediaEvent)
		{
			mediaEvent->Release();
			mediaEvent = NULL;
		}
		if (mediaControl)
		{
			mediaControl->Release();
			mediaControl = NULL;
		}
		graphBuilder.Release();
		graphBuilder = NULL;
	}

	STDMETHOD(Spustit)()
	{
		if (mediaControl)
			mediaControl->Run();
		return S_OK;
	}

	STDMETHOD(Zastavit)()
	{
		if (mediaControl)
			mediaControl->Stop();
		Fire_Zastaveno();
		return S_OK;
	}

	STDMETHOD(Prerusit)()
	{
		if (mediaControl)
			mediaControl->Pause();
		return S_OK;
	}

	STDMETHOD(Zavrit)()
	{
		if (!graphBuilder)
			return S_OK;
		if (mediaControl)
			mediaControl->StopWhenReady();
		if (videoWindow)
		{
			videoWindow->put_Visible(OAFALSE);
			videoWindow->put_Owner(NULL);
			videoWindow->Release();
			videoWindow = NULL;
		}
		if (basicAudio)
		{
			basicAudio->Release();
			basicAudio = NULL;
		}
		if (basicVideo)
		{
			basicVideo->Release();
			basicVideo = NULL;
		}
		ZrusitGraf();
		FireViewChange();
		Fire_Zastaveno();
		return S_OK;
	}

	STDMETHOD(VybratSoubor)(BSTR* soubor)
	{
		wchar_t szSoubor[MAX_PATH] = {0};
		OPENFILENAME ofn;
		memset(&ofn, 0, sizeof(OPENFILENAME));
		ofn.lStructSize = sizeof(OPENFILENAME);
		ofn.hwndOwner = m_hWnd;
		ofn.lpstrFile = szSoubor;
		ofn.nMaxFile = sizeof(szSoubor);
		ofn.lpstrFileTitle = NULL;
		ofn.nMaxFileTitle = 0;
		ofn.lpstrInitialDir = NULL;
		ofn.lpstrFilter = L"Multimediln soubory\0"
			L"*.wmv;*.wma;*.avi;*.wav;*.mpeg\0";
		ofn.Flags = OFN_FILEMUSTEXIST;
		if (!GetOpenFileName(&ofn)) 
			return S_FALSE;
		Zavrit();
		this->soubor = CComBSTR(szSoubor);
		Otevrit(this->soubor.m_str);
		this->soubor.CopyTo(soubor);
		return S_OK;
	}

	LRESULT OnLButtonDown(UINT uMsg,
		WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if (!tazeniMysi)
			return DefWindowProc();
		HWND hParent = ::GetParent(m_hWnd);
		if (!hParent)
			return DefWindowProc();
		while (true)
		{
			if (::GetParent(hParent))
				hParent = ::GetParent(hParent);
			else
				break;
		}
		::SendMessage(hParent, WM_SYSCOMMAND,
			SC_MOVE | HTCAPTION, 0);
		return DefWindowProc();
	}

	STDMETHOD(get_TazeniMysi)(VARIANT_BOOL* pHodnota)
	{
		*pHodnota = (VARIANT_BOOL)tazeniMysi;
		return S_OK;
	}

	STDMETHOD(put_TazeniMysi)(VARIANT_BOOL hodnota)
	{
		tazeniMysi = (hodnota != VARIANT_FALSE);
		return S_OK;
	}

	STDMETHOD(get_ObsahujeVideo)(VARIANT_BOOL* pHodnota)
	{
		*pHodnota = (VARIANT_BOOL)obsahujeVideo;
		return S_OK;
	}

	STDMETHOD(get_Soubor)(BSTR* pSoubor)
	{
		this->soubor.CopyTo(pSoubor);
		return S_OK;
	}
};

OBJECT_ENTRY_AUTO(__uuidof(MediaplayerAx), MediaplayerAx)
