/****************************************************
Rozen kontextov nabdky Przkumnka
 o nhled obrzku
Soubor: ObrShellExt.h
(C) 2005 Radek Chalupa - www.radekchalupa.cz
*****************************************************/

#pragma once

#include "resource.h"
#include "RozsireniShellu.h"


// ObrShellExt
class ATL_NO_VTABLE ObrShellExt : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<ObrShellExt, &CLSID_ObrShellExt>,
	public IShellExtInit,
	public IContextMenu3,
	public IDispatchImpl<IObrShellExt, &IID_IObrShellExt,
	&LIBID_RozsireniShelluLib, 1, 0>
{
private:
	wchar_t soubor[MAX_PATH];
	wchar_t infoText[300];

public:

	DECLARE_REGISTRY_RESOURCEID(IDR_OBRSHELLEXT)
	DECLARE_PROTECT_FINAL_CONSTRUCT()


BEGIN_COM_MAP(ObrShellExt)
	COM_INTERFACE_ENTRY(IObrShellExt)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IShellExtInit)
	COM_INTERFACE_ENTRY(IContextMenu)
	COM_INTERFACE_ENTRY(IContextMenu2)
	COM_INTERFACE_ENTRY(IContextMenu3)
END_COM_MAP()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease() 
	{
	}

	STDMETHOD(Initialize)(LPCITEMIDLIST pidlFolder,
		LPDATAOBJECT lpdobj, HKEY hkeyProgID)
	{
		if (lpdobj == NULL)
			return E_INVALIDARG;
		STGMEDIUM medium;
		FORMATETC fe = {CF_HDROP, NULL,
			DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
		HRESULT hr = lpdobj->GetData(&fe, &medium);
		if (FAILED(hr))
			return E_INVALIDARG;
		DragQueryFile(reinterpret_cast<HDROP>(medium.hGlobal),
			0, this->soubor, MAX_PATH);
		ATLTRACE2("Initialize: ");
		ATLTRACE2(this->soubor);
		ATLTRACE2("\n");
		ReleaseStgMedium(&medium);
		Bitmap bitmap(this->soubor);
		if (bitmap.GetLastStatus() != Ok)
			return hr;
		DWORD velikost;
		HANDLE hSoubor = CreateFile(this->soubor,
			0, FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING,
			0,NULL);
		wchar_t szVelikost[40];
		if (hSoubor == INVALID_HANDLE_VALUE)
			wcscpy(szVelikost, L"neznm?");
		else
		{
			velikost = GetFileSize(hSoubor, NULL);
			CloseHandle(hSoubor);
			StrFormatByteSizeW(velikost, szVelikost,
				sizeof(szVelikost)/sizeof(wchar_t));
		}
		swprintf(infoText, L"Obrzek: %d x %d,"
			L" Velikost: %s"
			L" -> zobrazit v prohlei",
			bitmap.GetWidth(),
			bitmap.GetHeight(),
			szVelikost);
		return hr;
	}

	STDMETHOD(QueryContextMenu)(HMENU hmenu,
		UINT indexMenu,
		UINT idCmdFirst, UINT idCmdLast,
		UINT uFlags)
	{
		if (!(CMF_DEFAULTONLY & uFlags))
		{
			MENUITEMINFO mii;
			memset(&mii, 0, sizeof(MENUITEMINFO));
			mii.cbSize = sizeof(MENUITEMINFO);
			mii.fMask = MIIM_STATE | MIIM_ID | MIIM_STRING | MIIM_FTYPE;
			mii.fState = MFS_ENABLED;
			mii.fType = MFT_OWNERDRAW;
			mii.fState = MFS_ENABLED;
			mii.dwTypeData = L"Nhled";
			mii.wID = idCmdFirst;
			InsertMenuItem(hmenu, indexMenu++, TRUE, &mii);

			InsertMenu(hmenu, indexMenu++,
				MF_SEPARATOR | MF_BYPOSITION,
				idCmdFirst + 1, NULL);
			return MAKE_HRESULT(SEVERITY_SUCCESS,
				FACILITY_NULL, 2);
		}
		return MAKE_HRESULT(SEVERITY_SUCCESS,
			FACILITY_NULL, USHORT(0));
	}

	STDMETHOD(GetCommandString)(UINT idCmd,
		UINT uFlags, UINT *pwReserved,
		LPSTR pszName, UINT cchMax)
	{
		if (uFlags & GCS_HELPTEXT)
		{
			// Windows NT will ask for Unicode strings
			switch(idCmd)
			{
			case 0:
				if (uFlags & GCS_VERBA)
					strncpy(pszName,
					"Zobraz obrzek v prohlei", cchMax);
				else
					lstrcpynW((wchar_t*)pszName,
					infoText,
					cchMax);
				ATLTRACE2("GetCommandString\n");
				break;
			}
			return S_OK;
		}
		return E_FAIL;
	}

	STDMETHOD(InvokeCommand)(LPCMINVOKECOMMANDINFO pici)
	{
		wchar_t parametry[MAX_PATH+6];
		UINT idOffset = LOWORD(pici->lpVerb);
		switch (idOffset)
		{	
			case 0:
				lstrcpy(parametry, L"/u \"");
				lstrcat(parametry, this->soubor);
				lstrcat(parametry, L"\"");
				ShellExecute(NULL, L"open", this->soubor,
					NULL, NULL, 0);
				break;
				
		}
		return S_OK;
	}

	STDMETHOD(HandleMenuMsg)(UINT uMsg,
		WPARAM wParam, LPARAM lParam)
	{
		LRESULT lres = 0;
		return HandleMenuMsg2(uMsg, wParam, lParam, &lres);
	}


	STDMETHOD(HandleMenuMsg2)(UINT uMsg,
		WPARAM wParam, LPARAM lParam,
		LRESULT *plResult)
	{
		switch (uMsg)
		{
		case WM_MEASUREITEM:
			OnMeasureItem((MEASUREITEMSTRUCT*)lParam);
			if (plResult)
				*plResult = TRUE;
			break;
		case WM_DRAWITEM:
			OnDrawItem((DRAWITEMSTRUCT*)lParam);
			if (plResult)
				*plResult = TRUE;
			break;
		}
		return S_OK;
	}

	void OnMeasureItem(MEASUREITEMSTRUCT* mis)
	{
		if (mis->itemWidth < 200)
			mis->itemWidth = 200;
		mis->itemHeight = 200;
		Bitmap bitmap(this->soubor);
		if (bitmap.GetLastStatus() != Ok)
			return;
		if (bitmap.GetWidth() > bitmap.GetHeight())
		{
			int vyska =
				(bitmap.GetHeight() * mis->itemWidth) /
				bitmap.GetWidth();
			if (vyska >= 32)
				mis->itemHeight = vyska;
		}
	}

	void OnDrawItem(DRAWITEMSTRUCT* dis)
	{
		Bitmap bitmap(this->soubor);
		if (bitmap.GetLastStatus() != Ok)
			return;
		if (bitmap.GetHeight() == 0)
			return;
		if (bitmap.GetWidth() == 0)
			return;
			
		int sirkaRect = dis->rcItem.right - dis->rcItem.left;
		int vyskaRect = dis->rcItem.bottom - dis->rcItem.top;
		int xPoz = dis->rcItem.left;
		int yPoz = dis->rcItem.top;
		int sirkaObrCil = bitmap.GetWidth();
		int vyskaObrCil = bitmap.GetHeight();

		RECT rcPozadi;
		Graphics gr(dis->hDC);
		HBRUSH hbPozadi;
		if ((dis->itemState & ODS_SELECTED ||
				dis->itemState & ODS_HOTLIGHT) 
				&& !(dis->itemState & ODS_DISABLED))
			hbPozadi = GetSysColorBrush(COLOR_HIGHLIGHT);
		else
			hbPozadi = GetSysColorBrush(COLOR_MENU);

		if (sirkaObrCil < sirkaRect && vyskaObrCil < vyskaRect)
		{
			xPoz = dis->rcItem.left + ((sirkaRect - sirkaObrCil) / 2);
			yPoz = dis->rcItem.top + ((vyskaRect - vyskaObrCil) / 2);
			if (hbPozadi)
			{
				// horn pruh
				rcPozadi.left = dis->rcItem.left;
				rcPozadi.top = dis->rcItem.top;
				rcPozadi.right = dis->rcItem.right;
				rcPozadi.bottom = yPoz;
				FillRect(dis->hDC, &rcPozadi, hbPozadi);
				// doln pruh
				rcPozadi.top = yPoz + vyskaObrCil;
				rcPozadi.bottom = dis->rcItem.bottom;
				FillRect(dis->hDC, &rcPozadi, hbPozadi);
				// lev pruh
				rcPozadi.left = dis->rcItem.left;
				rcPozadi.top = yPoz;
				rcPozadi.right = xPoz;
				rcPozadi.bottom = yPoz + vyskaObrCil;
				FillRect(dis->hDC, &rcPozadi, hbPozadi);
				// prav
				rcPozadi.left = xPoz + sirkaObrCil;
				rcPozadi.top = yPoz;
				rcPozadi.right = dis->rcItem.right;
				rcPozadi.bottom = yPoz + vyskaObrCil;
				FillRect(dis->hDC, &rcPozadi, hbPozadi);
			}
		}
		else
		{
			// nen li obrzek pesn do velikosti
			if (sirkaRect < sirkaObrCil || vyskaRect < vyskaObrCil)
			{
				if ((float)sirkaRect/(float)sirkaObrCil >
					(float)vyskaRect/(float)vyskaObrCil)
				{ // obrzek "na vku", pruhy pozad vlevo a vpravo
					sirkaObrCil =
						(bitmap.GetWidth() * vyskaRect)
						/ bitmap.GetHeight();
					vyskaObrCil = vyskaRect;
					xPoz = dis->rcItem.left +
						((sirkaRect - sirkaObrCil) / 2);
					if (hbPozadi)
					{
						rcPozadi.left = dis->rcItem.left;
						rcPozadi.top = dis->rcItem.top;
						rcPozadi.right = xPoz;
						rcPozadi.bottom = dis->rcItem.bottom;
						FillRect(dis->hDC, &rcPozadi, hbPozadi);
						// prav pruh pozad
						rcPozadi.right = dis->rcItem.right;
						rcPozadi.left = xPoz + sirkaObrCil;
						FillRect(dis->hDC, &rcPozadi, hbPozadi);
					}
				}
				else
				{ // obrzek na ku, pruhy pozad nahoe a dole
					vyskaObrCil =
						(bitmap.GetHeight() * sirkaRect) /
						bitmap.GetWidth();
					sirkaObrCil = sirkaRect;
					yPoz = dis->rcItem.top +
						((vyskaRect - vyskaObrCil) / 2);
					if (hbPozadi)
					{
						rcPozadi.left = dis->rcItem.left;
						rcPozadi.right = dis->rcItem.right;
						rcPozadi.top = dis->rcItem.top;
						rcPozadi.bottom = yPoz;
						FillRect(dis->hDC, &rcPozadi, hbPozadi);
						rcPozadi.top = yPoz + vyskaObrCil;
						rcPozadi.bottom = dis->rcItem.bottom;
						FillRect(dis->hDC, &rcPozadi, hbPozadi);
					}
				}
			}
			else
			{
				sirkaObrCil = sirkaRect;
				vyskaObrCil = vyskaRect;
			}
		}
		Rect rectCilObr(xPoz, yPoz,	sirkaObrCil, vyskaObrCil);
		gr.DrawImage(&bitmap,
			rectCilObr,
			0,
			0,
			bitmap.GetWidth(),
			bitmap.GetHeight(),
			UnitPixel);
	}
};

OBJECT_ENTRY_AUTO(__uuidof(ObrShellExt), ObrShellExt)
